/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIAYSTOK 2014                               **
**                                                                        **
**                         Home Control System                            **
**                                                                        **
**                              HC_TMPS1                                  **
**                      Wersja z dnia : 2018.06.20                        **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "tmps1eeprom_ds1820.h"
#include <avr/wdt.h>


#define false                          0
#define true                           1

#define TimerDelay480us                3539
#define TimerDelay150us                1106
#define TimerDelay5us                  7
#define TimerDelay15us                 20
#define TimerDelay45us                 64
#define TimerDelay60us                 89
#define _1WireDelay1us() __asm__ __volatile__ ("nop\n" \
                                           "nop\n" \
                                           "nop\n" \
                                           "nop\n" \
                                           "nop\n" \
                                           "nop" )

#define _1WireTimeMaxDelay             2500   //1800
#define _1WirePort                     PORTB
#define _1WireDirPort                  DDRB
#define _1WirePin                      PINB
#define _1WireDataPin                  0              // PORTB.0

#define Set_1WireAsInput               _1WireDirPort &= ~ ( ( uint8_t ) 1 << _1WireDataPin )
#define Set_1WireAsOutput              _1WireDirPort |= ( uint8_t ) 1 << _1WireDataPin
#define Set_1WireLow                   _1WirePort &= ~ ( ( uint8_t ) 1 << _1WireDataPin )
#define Set_1WireHigh                  _1WirePort |= ( uint8_t ) 1 << _1WireDataPin
#define Get_1WirePin                   ( ( _1WirePin ) & ( ( uint8_t ) 1 << _1WireDataPin ) )

#define StatusLed                      3
#define RxTxLed                        2
#define SensorFailLed                  1
#define ConnectLed                     0

#define TimerCounterDelayLimit         3000  // 2048
#define StandardLEDPulseDelay          832

#define LEDPort                        PORTC
#define LEDPortDirection               DDRC

#define null                           0
#define cr                             0x0D
#define Space                          0x20
#define PartSeparator                  ':'
#define ParamSeparator                 '.'

#define nop() __asm__ __volatile__ ("nop")

// USART BAUD Rate Generator fosc=7.3728 MHz
#define Speed9600bps                   47
#define Speed19200bps                  23
#define Speed38400bps                  11
#define Speed115200bps                 3

#define RxTxDirPort                    PORTD
#define RxTxDirDirectionPort           DDRD
#define RxTxDirPin                     2              // PORTD . 2

#define IdleState                      0
#define BusyState                      1

#define TXDBuffSize                    128
#define RXDBuffSize                    64

typedef struct { uint8_t CycFr ;
                 uint8_t CycTo ;
                 uint8_t State ;
                 uint8_t Buffer [ TXDBuffSize ] ;
               } TXDRecT ;
typedef struct { uint8_t CycFr ;
                 uint8_t CycTo ;
                 uint8_t Buffer [ RXDBuffSize ] ;
               } RXDRecT ;

#define CyclBuffSize                   250

typedef struct { uint8_t CycFr ;
                 uint8_t CycTo ;
                 uint8_t Buffer [ CyclBuffSize ] ;
               } CyclicRecT ;

typedef void ( * OperationProcT ) ( void ) ;

typedef struct {
                 volatile TXDRecT          TXDRec ;
                 volatile RXDRecT          RXDRec ;
                 OperationProcT            SerialService ;
                 CommandBufferType         CommandText ;
                 ParamBufferType           ParamText ;
                 DeviceIDType              DestinationDeviceName ;
                 DeviceIDType              SourceDeviceName ;
                 uint8_t                   CommandTextIndex ;
                 uint8_t                   ParamTextIndex ;
                 uint8_t                   DestDeviceNameIndex ;
                 uint8_t                   SrcDeviceNameIndex ;
                 uint16_t                  SerialLEDDelay ;
               } ServiceType ;

typedef ServiceType * PtrServiceType ;

typedef enum { KeybIdle ,
               KeybWaitOnPress ,
               KeybReady ,
               KeybActive ,
               KeybWaitOnFree ,
               KeybWait
             } KeybStatusType ;

typedef void ( * CommandExecuteServiceProcT ) ( void ) ;
typedef struct {
                 CommandExecuteServiceProcT  CommandService ;
                 uint8_t *                   CommandName ;
               } CommandNameAssociationRecT ;

#define SensorTimeMaxDelay             444
#define TempEchoDevicesArraySize       4
#define ParamAsciiValueSize            6

static volatile uint8_t TimeEvent ;
static volatile uint8_t _1WireEvent ;
static volatile uint16_t _1WireTimeDelay ;
static ServiceType SerialService ;
static uint8_t ConnectContext ;
static uint16_t TimerCounter ;
static DeviceIDType TempEchoDevices [ TempEchoDevicesArraySize ] ;
static CyclicRecT CyclicBuffer ;
static int16_t CurrentTemp ;
static uint8_t _1WireAutomatState ;
static uint8_t SensorPresent ;
static uint8_t TempDataValid ;
static uint8_t ParamAsciiValue [ ParamAsciiValueSize + 1 ] ;
static uint8_t ParamAsciiValueIndex ;
static uint8_t CurrentMode ;
static uint16_t AutoDelayCounter ;
static CommandExecuteServiceProcT AutoSendService ;



static const uint16_t CRCTable [ 16 ] = { 0x0000 ,  /* 00 */
                                          0xCC01 ,  /* 01 */
                                          0xD801 ,  /* 02 */
                                          0x1400 ,  /* 03 */
                                          0xF001 ,  /* 04 */
                                          0x3C00 ,  /* 05 */
                                          0x2800 ,  /* 06 */
                                          0xE401 ,  /* 07 */
                                          0xA001 ,  /* 08 */
                                          0x6C00 ,  /* 09 */
                                          0x7800 ,  /* 10 */
                                          0xB401 ,  /* 11 */
                                          0x5000 ,  /* 12 */
                                          0x9C01 ,  /* 13 */
                                          0x8801 ,  /* 14 */
                                          0x4400 } ;/* 15 */

static uint8_t HardwareSoftwareVersion [ ] PROGMEM   = "HVER*1_00*V-2013.SVER*1_1*VI-2018\r" ;

static uint8_t DeviceName [ ] PROGMEM                = "HCTMPS1-temp server (DS18B20)\r" ;
static uint8_t GeneralPoolDeviceName [ ] PROGMEM     = "GENERAL" ;
static uint8_t OK [ ] PROGMEM                        = "<OK>\r" ;
static uint8_t Fail [ ] PROGMEM                      = "<FA>\r" ;
static uint8_t HexalTable [ ] PROGMEM                = "0123456789ABCDEF" ;
static uint8_t NoDataText [ ] PROGMEM                = "NODATA\r" ;
static uint8_t EmptyFlagTxt [ ] PROGMEM              = "<EMPTY>\r" ;
static uint8_t ModeIsOnTxt [ ] PROGMEM               = "MDISON" ;
static uint8_t ModeIsOffTxt [ ] PROGMEM              = "MDISOF" ;
static uint8_t AutoSendCommand [ ] PROGMEM           = "RPV" ;

static uint8_t ReplyCommand [ ] PROGMEM              = "REPLY" ;
static uint8_t ConnectCommand [ ] PROGMEM            = "CONNECT" ;
static uint8_t DisConnectCommand [ ] PROGMEM         = "DISCONNECT" ;
static uint8_t SetDeviceNameCommand [ ] PROGMEM      = "SETMID" ;
static uint8_t HelloCommand [ ] PROGMEM              = "HELLO" ;
static uint8_t GetVersionCommand [ ] PROGMEM         = "GETVER" ;
static uint8_t GetIDCommand [ ] PROGMEM              = "GETID" ;
static uint8_t ReinitCommand [ ] PROGMEM             = "REINIT" ;
static uint8_t ResetEECommand [ ] PROGMEM            = "RESEE" ;
static uint8_t SaveEECommand [ ] PROGMEM             = "SAVEEE" ;
static uint8_t AddSuperiorChnCommand [ ] PROGMEM     = "ADDSUP" ;
static uint8_t DelSuperiorChnCommand [ ] PROGMEM     = "DELSUP" ;
static uint8_t ListSuperiorChnCommand [ ] PROGMEM    = "LISTSUP" ;
static uint8_t NewPasswordCommand [ ] PROGMEM        = "NEWPW" ;
static uint8_t AddParamCommand [ ] PROGMEM           = "ADDP" ;
static uint8_t DelParamCommand [ ] PROGMEM           = "DELP" ;
static uint8_t ListParamCommand [ ] PROGMEM          = "LISTP" ;
static uint8_t GetParamValueCommand [ ] PROGMEM      = "PV" ;
static uint8_t MeasServiceOnCommand [ ] PROGMEM      = "CSSERVON" ;
static uint8_t MeasServiceOffCommand [ ] PROGMEM     = "CSSERVOFF" ;
static uint8_t MeasServiceListCommand [ ] PROGMEM    = "CSSERVLIST" ;
static uint8_t AutoOnCommand [ ] PROGMEM             = "AUTOON" ;
static uint8_t AutoOffCommand [ ] PROGMEM            = "AUTOOFF" ;
static uint8_t GetModeCommand [ ] PROGMEM            = "GETMODE" ;

static void ReplyCommandExecute ( void ) ;
static void ConnectCommandExecute ( void ) ;
static void DisConnectCommandExecute ( void ) ;
static void SetDeviceNameCommandExecute ( void ) ;
static void HelloCommandExecute ( void ) ;
static void GetVersionCommandExecute ( void ) ;
static void GetIDCommandExecute ( void ) ;
static void ReinitCommandExecute ( void ) ;
static void ResetEECommandExecute ( void ) ;
static void SaveEECommandExecute ( void ) ;
static void AddSuperiorChnCommandExecute ( void ) ;
static void DelSuperiorChnCommandExecute ( void ) ;
static void ListSuperiorChnCommandExecute ( void ) ;
static void NewPasswordCommandExecute ( void ) ;
static void AddParamCommandExecute ( void ) ;
static void DelParamCommandExecute ( void ) ;
static void ListParamCommandExecute ( void ) ;
static void GetParamValueCommandExecute ( void ) ;
static void MeasServiceOnCommandExecute ( void ) ;
static void MeasServiceOffCommandExecute ( void ) ;
static void MeasServiceListCommandExecute ( void ) ;
static void AutoOnCommandExecute ( void ) ;
static void AutoOffCommandExecute ( void ) ;
static void GetModeCommandExecute ( void ) ;


#define CommandNameAssociationTabeSize               24

static CommandNameAssociationRecT CommandNameAssociationTabe [ CommandNameAssociationTabeSize ] PROGMEM =
{
/* 00 */ { ReplyCommandExecute                 , ReplyCommand } ,
/* 01 */ { ConnectCommandExecute               , ConnectCommand } ,
/* 02 */ { DisConnectCommandExecute            , DisConnectCommand } ,
/* 03 */ { SetDeviceNameCommandExecute         , SetDeviceNameCommand } ,
/* 04 */ { HelloCommandExecute                 , HelloCommand } ,
/* 05 */ { GetVersionCommandExecute            , GetVersionCommand } ,
/* 06 */ { GetIDCommandExecute                 , GetIDCommand } ,
/* 07 */ { ReinitCommandExecute                , ReinitCommand } ,
/* 08 */ { ResetEECommandExecute               , ResetEECommand } ,
/* 09 */ { SaveEECommandExecute                , SaveEECommand } ,
/* 10 */ { AddSuperiorChnCommandExecute        , AddSuperiorChnCommand } ,
/* 11 */ { DelSuperiorChnCommandExecute        , DelSuperiorChnCommand } ,
/* 12 */ { ListSuperiorChnCommandExecute       , ListSuperiorChnCommand } ,
/* 13 */ { NewPasswordCommandExecute           , NewPasswordCommand } ,
/* 14 */ { AddParamCommandExecute              , AddParamCommand } ,
/* 15 */ { DelParamCommandExecute              , DelParamCommand } ,
/* 16 */ { ListParamCommandExecute             , ListParamCommand } ,
/* 17 */ { GetParamValueCommandExecute         , GetParamValueCommand } ,
/* 18 */ { MeasServiceOnCommandExecute         , MeasServiceOnCommand } ,
/* 19 */ { MeasServiceOffCommandExecute        , MeasServiceOffCommand } ,
/* 20 */ { MeasServiceListCommandExecute       , MeasServiceListCommand } ,
/* 21 */ { AutoOnCommandExecute                , AutoOnCommand } ,
/* 22 */ { AutoOffCommandExecute               , AutoOffCommand } ,
/* 23 */ { GetModeCommandExecute               , GetModeCommand } ,
} ;



static void USARTReceiveMode ( void )
{
  /*----------------------------------------------------------------*/
  PORTD &= ~ ( 1 << RxTxDirPin ) ;
} /* USARTReceiveMode */


static void USARTTransmitMode ( void )
{
  /*----------------------------------------------------------------*/
  PORTD |= ( 1 << RxTxDirPin ) ;
} /* USARTTransmitMode */


SIGNAL ( TIMER0_OVF_vect )
{
  /*----------------------------------------------------------------*/
  TimerCounter ++ ;
  if ( TimerCounter > TimerCounterDelayLimit )
  {
    TimerCounter = 0 ;
    TimeEvent = true ;
  } /* if */ ;
  if ( SerialService . SerialLEDDelay )
  {
    SerialService . SerialLEDDelay -- ;
    if ( ! SerialService . SerialLEDDelay )
    {
      LEDPort |= ( 1 << RxTxLed ) ;
    } /* if */ ;
  } /* if */ ;
  _1WireTimeDelay ++ ;
  if ( _1WireTimeDelay > _1WireTimeMaxDelay )
  {
    _1WireEvent = 1 ;
    _1WireTimeDelay = 0 ;
  } /* if */ ;
} /* TIMER0_OVF_vect */


static void StartRxTxLED ( void )
{
  /*----------------------------------------------------------------*/
  LEDPort &= ~ ( 1 << RxTxLed ) ;
  SerialService . SerialLEDDelay = StandardLEDPulseDelay ;
} /* StartRxTxLED */


SIGNAL ( USART_RX_vect )
{
  uint8_t RecCh ;
  /*----------------------------------------------------------------*/
  RecCh = UDR0 ;
  SerialService . RXDRec . Buffer [ SerialService . RXDRec . CycTo ] = RecCh ;
  SerialService . RXDRec . CycTo ++ ;
  if ( SerialService . RXDRec . CycTo >= RXDBuffSize )
    SerialService . RXDRec . CycTo = 0 ;
  StartRxTxLED ( ) ;
} /* USART_RX_vect */


SIGNAL ( USART_TX_vect )
{
  /*----------------------------------------------------------------*/
  if ( SerialService . TXDRec . CycFr == SerialService . TXDRec . CycTo )
  {
    SerialService . TXDRec . State = IdleState ;
    USARTReceiveMode ( ) ;
  } /* if ... */
  else
  {
    UDR0 = SerialService . TXDRec . Buffer [ SerialService . TXDRec . CycFr ] ;
    SerialService . TXDRec . CycFr ++ ;
    if ( SerialService . TXDRec . CycFr >= TXDBuffSize )
      SerialService . TXDRec . CycFr = 0 ;
    StartRxTxLED ( ) ;
  } /* if ... else */ ;
} /* USART_TX_vect */


static void SendSerial ( uint8_t SerialCh )
{
  uint8_t NextInx ;
  /*----------------------------------------------------------------*/
  USARTTransmitMode ( ) ;
  StartRxTxLED ( ) ;
  cli ( ) ;
  if ( SerialService . TXDRec . State == IdleState )
  {
    SerialService . TXDRec . State = BusyState ;
    UDR0 = SerialCh ;
    sei ( ) ;
  } /* if ... */
  else
  {
    for ( ; ; )
    {
      NextInx = SerialService . TXDRec . CycTo + 1 ;
      if ( NextInx >= TXDBuffSize )
        NextInx = 0 ;
      if ( NextInx != SerialService . TXDRec . CycFr )
        break ;
      sei ( ) ;
      nop ( ) ;
      nop ( ) ;
      wdt_reset ( ) ;
      cli ( ) ;
    } /* for */ ;
    SerialService . TXDRec . Buffer [ SerialService . TXDRec . CycTo ] = SerialCh ;
    SerialService . TXDRec . CycTo ++ ;
    if ( SerialService . TXDRec . CycTo >= TXDBuffSize )
      SerialService . TXDRec . CycTo = 0 ;
    sei ( ) ;
  } /* if ... else */ ;
} /* SendSerial */


static void SendSerialTxt ( uint8_t * String )
{
  uint8_t Data ;
  /*----------------------------------------------------------------*/
  for ( ; ; )
  {
    Data = * String ++ ;
    if ( Data )
      SendSerial ( Data ) ;
    else
      break ;
  } /* for */ ;
} /* SendSerialTxt */


static void SendSerialFlashTxt ( uint16_t String )
{
  uint8_t Data ;
  /*----------------------------------------------------------------*/
  for ( ; ; )
  {
    Data = pgm_read_byte_near ( String ) ;
    if ( Data )
      SendSerial ( Data ) ;
    else
      break ;
    String ++ ;
  } /* for */ ;
} /* SendSerialFlashTxt */


static uint8_t SerialDataPresent ( void )
{
  if ( SerialService . RXDRec . CycFr == SerialService . RXDRec . CycTo )
    return ( false ) ;
  else
    return ( true ) ;
} /* SerialDataPresent */


static uint8_t GetSerial ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  Ch = SerialService . RXDRec . Buffer [ SerialService . RXDRec . CycFr ] ;
  SerialService . RXDRec . CycFr ++ ;
  if ( SerialService . RXDRec . CycFr >= RXDBuffSize )
    SerialService . RXDRec . CycFr = 0 ;
  return ( Ch ) ;
} /* GetSerial */


static void AddToCyclic ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
  CyclicBuffer . Buffer [ CyclicBuffer . CycTo ] = Ch ;
  CyclicBuffer . CycTo ++ ;
  if ( CyclicBuffer . CycTo >= CyclBuffSize )
    CyclicBuffer. CycTo = 0 ;
} /* AddToCyclic */


static void AddToCyclicTxt ( uint8_t * String )
{
  uint8_t Data ;
  /*----------------------------------------------------------------*/
  for ( ; ; )
  {
    Data = * String ++ ;
    if ( Data )
      AddToCyclic ( Data ) ;
    else
      break ;
  } /* for */ ;
} /* AddToCyclicTxt */


static void AddToCyclicFlashTxt ( uint16_t String )
{
  uint8_t Data ;
  /*------------------------------*/
  for ( ; ; )
  {
    Data = pgm_read_byte_near ( String ) ;
    if ( Data )
      AddToCyclic ( Data ) ;
    else
      break ;
    String ++ ;
  } /* for */ ;
} /* AddToCyclicFlashTxt */


static uint8_t CyclicDataPresent ( void )
{
  /*----------------------------------------------------------------*/
  if ( CyclicBuffer . CycFr == CyclicBuffer . CycTo )
    return ( false ) ;
  else
    return ( true ) ;
} /* CyclicDataPresent */


static uint8_t GetFromCyclic ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  Ch = CyclicBuffer . Buffer [ CyclicBuffer . CycFr ] ;
  CyclicBuffer . CycFr ++ ;
  if ( CyclicBuffer . CycFr >= CyclBuffSize )
    CyclicBuffer . CycFr = 0 ;
  return ( Ch ) ;
} /* GetFromCyclic */


static void FlushCyclic ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  while ( CyclicDataPresent ( ) )
  {
    Ch = GetFromCyclic ( );
    SendSerial ( Ch ) ;
  } /* while */ ;
} /* FlushCyclic */


static void USARTHardwareInit ( uint16_t Speed )
{
  /*----------------------------------------------------------------*/
  UCSR0A = 0 ;
  UCSR0B = ( 1 << RXCIE0 ) | ( 1 << TXCIE0 ) | ( 1 << RXEN0 ) | ( 1 << TXEN0 ) ;
  UCSR0C = ( 1 << UCSZ01 ) | ( 1 << UCSZ00 ) ;
  UBRR0 = Speed ;
} /* USARTHardwareInit */


static void HardwareInit ( uint16_t Speed )
{
  /*----------------------------------------------------------------*/
  TCCR0A = 0 ;
  TCCR0B = ( 1 << CS01 ) ; // CS02=0 CS01=1 CS00=0 : F clk div 8
  TCNT0 = 0 ;
  OCR0A = 0 ;
  OCR0B = 0 ;
  TIMSK0 = ( 1 << TOIE0 ) ;
  TCCR1A = 0 ;
  TCCR1B = ( 1 << CS11 ) ; // CS12=0 CS11=1 CS10=0 : F clk div 8
  TCCR1C = 0 ;
  TCNT1 = 0 ;
  USARTHardwareInit ( Speed ) ;
} /* HardwareInit */


static void EnvirInit ( void )
{
  uint8_t Bits ;
  /*----------------------------------------------------------------*/
  Bits = ( 1 << StatusLed ) | ( 1 << RxTxLed ) | ( 1 << SensorFailLed ) | ( 1 << ConnectLed ) ;
  LEDPortDirection |= Bits ;
  LEDPort |= Bits ;
  RxTxDirDirectionPort |= ( 1 << RxTxDirPin ) ;
  Set_1WireHigh ;
  Set_1WireAsInput ;
} /* EnvirInit */


static void ClearBlock ( void *   Destination ,
                         uint16_t Size )
{
  uint16_t Loop ;
  uint8_t * Dst ;
  /*----------------------------------------------------------------*/
  Dst = ( uint8_t * ) Destination ;
  for ( Loop = 0 ; Loop < Size ; Loop ++ )
    * Dst ++ = null ;
} /* ClearBlock */


static void StrCopy ( uint8_t * Destination ,
                      uint8_t * Source )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  Ch = * Source ;
  * Destination = Ch ;
  while ( Ch )
  {
    Ch = * Source ++ ;
    * Destination ++ = Ch ;
  } /* while */ ;
} /* StrCopy */


static void Assign ( uint8_t * SubDeviceID )
{
  uint8_t Loop ;
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  ClearBlock ( SubDeviceID , sizeof ( DeviceIDType ) ) ;
  for ( Loop = 0 ; Loop < DeviceIDSize ; Loop ++ )
  {
    Ch = SerialService . ParamText [ Loop ] ;
    if ( ! Ch )
      break ;
    * SubDeviceID ++ = Ch ;
  } /* for */ ;
} /* Assign */


static uint8_t CompareFlash ( uint8_t * Text ,
                              uint16_t  Pattern )
{
  uint8_t Ch1 ;
  uint8_t Ch2 ;
  /*----------------------------------------------------------------*/
  for ( ; ; )
  {
    Ch1 = * Text ++ ;
    Ch2 = pgm_read_byte_near ( Pattern ++ ) ;
    if ( Ch1 != Ch2 )
    {
      return ( false ) ;
    } /* if */ ;
    if ( Ch1 == 0 )
    {
      return ( true ) ;
    } /* if */ ;
  } /* for */ ;
} /* CompareFlash */


static uint8_t CompareDeviceID ( uint8_t * DeviceName ,
                                 uint8_t * Pattern )
{
  uint8_t Ch1 ;
  uint8_t Ch2 ;
  /*----------------------------------------------------------------*/
  for ( ; ; )
  {
    Ch1 = * DeviceName ++ ;
    Ch2 = * Pattern ++ ;
    if ( Ch1 != Ch2 )
    {
      return ( false ) ;
    } /* if */ ;
    if ( ( Ch1 == null ) || ( Ch2 == null ) )
      return ( true ) ;
  } /* for */ ;
} /* CompareDeviceID */


static void CodePassword ( void )
{
  uint8_t Loop ;
  uint8_t Element ;
  uint8_t BitCorrection ;
  /*----------------------------------------------------------------*/
  for ( Loop = 0 ; Loop <= PasswordSize ; Loop ++ )
  {
    Element = EEBlock . Password [ Loop ] ^ 0xFF ;
    BitCorrection = Element & 0x01 ;
    Element = Element >> 1 ;
    if ( BitCorrection )
      Element |= 0x80 ;
    EEBlock . Password [ Loop ] = Element ;
  } /* for */ ;
} /* CodePassword */


static void DecodePassword ( void )
{
  uint8_t Loop ;
  uint16_t Element ;
  /*----------------------------------------------------------------*/
  for ( Loop = 0 ; Loop <= PasswordSize ; Loop ++ )
  {
    Element = ( uint16_t ) EEBlock . Password [ Loop ] ;
    Element = Element << 1 ;
    if ( Element >= 0x100 )
      Element ++ ;
    Element &= 0xFF ;
    Element ^= 0xFF ;
    EEBlock . Password [ Loop ] = ( uint8_t ) Element ;
  } /* for */ ;
} /* DecodePassword */


static uint16_t ComputeCRC ( uint16_t  Length ,
                             uint8_t * Buffer )
{
  uint16_t Loop ;
  uint16_t ResultCRC ;
  uint16_t WorkCRC ;
  /*----------------------------------------------------------------*/
  ResultCRC = 0xFFFF ;
  for ( Loop = 0 ; Loop < Length ; Loop ++ )
  {
    WorkCRC = CRCTable [ ResultCRC & 0x0F ] ;
    ResultCRC = ( ResultCRC >> 4 ) & 0x0FFF ;
    ResultCRC ^= WorkCRC ^ CRCTable [ Buffer [ Loop ] & 0x0F ] ;
    WorkCRC = CRCTable [ ResultCRC & 0x0F ] ;
    ResultCRC = ( ResultCRC >> 4 ) & 0x0FFF ;
    ResultCRC ^= WorkCRC ^ CRCTable [ ( Buffer [ Loop ] >> 4 ) & 0x0F ] ;
  } /* for */ ;
  return ( ResultCRC ) ;
} /* ComputeCRC */


static void ReadEEBlock ( void )
{
  /*----------------------------------------------------------------*/
  eeprom_read_block ( & EEBlock , ( void * ) EEBlockAddressOffset ,
                      sizeof ( EEPROMBlockDataRecT ) ) ;
} /* ReadEEBlock */


static void WriteEEBlock ( void )
{
  uint16_t LocalCRCValue ;
  /*----------------------------------------------------------------*/
  CodePassword ( ) ;
  wdt_disable ( ) ;
  EEBlock . CRCValue = 0 ;
  LocalCRCValue = ComputeCRC ( sizeof ( EEPROMBlockDataRecT ) -
                               sizeof ( uint16_t ) ,
                               ( uint8_t * ) ( & EEBlock ) ) ;
  EEBlock . CRCValue = LocalCRCValue ;
  eeprom_write_block ( & EEBlock , ( void * ) EEBlockAddressOffset ,
                       sizeof ( EEPROMBlockDataRecT ) ) ;
  DecodePassword ( ) ;
  wdt_enable ( WDTO_1S ) ;
} /* WriteEEBlock */


static void EEBlockOperation ( void )
{
  uint16_t CRCValue ;
  /*----------------------------------------------------------------*/
  ReadEEBlock ( ) ;
  if ( EEBlock . EEBlockMetric != StandardSygnature )
  {
    SetDefaultEEBlock ( ) ;
    WriteEEBlock ( ) ;
    return ;
  } /* if */ ;
  CRCValue = ComputeCRC ( sizeof ( EEPROMBlockDataRecT ) ,
                          ( uint8_t * ) ( & EEBlock ) ) ;
  DecodePassword ( ) ;
  if ( CRCValue != 0 )
  {
    SetDefaultEEBlock ( ) ;
    WriteEEBlock ( ) ;
  } /* if */ ;
} /* EEBlockOperation */


static uint8_t MeDestination ( void )
{
  /*----------------------------------------------------------------*/
  return ( CompareDeviceID ( SerialService . DestinationDeviceName , EEBlock . MyDeviceID ) ) ;
} /* MeDestination */


static uint8_t GeneralDestination ( void )
{
  /*----------------------------------------------------------------*/
  return ( CompareFlash ( SerialService . DestinationDeviceName , ( uint16_t ) GeneralPoolDeviceName ) ) ;
} /* GeneralDestination */


static void AddToCyclicReplyDevicesID ( void )
{
  /*----------------------------------------------------------------*/
  AddToCyclicTxt ( SerialService . SourceDeviceName ) ;
  AddToCyclic ( PartSeparator ) ;
  AddToCyclicTxt ( SerialService . DestinationDeviceName ) ;
  AddToCyclic ( PartSeparator ) ;
} /* AddToCyclicReplyDevicesID */


static void AddToCyclicReplyForCommand ( void )
{
  /*----------------------------------------------------------------*/
  AddToCyclic ( 'R' ) ;
  AddToCyclicTxt ( SerialService . CommandText ) ;
  AddToCyclic ( ParamSeparator ) ;
} /* AddToCyclicReplyForCommand */


static void AddToCyclicCommandReplyOK ( void )
{
  /*----------------------------------------------------------------*/
  AddToCyclicReplyDevicesID ( ) ;
  AddToCyclicReplyForCommand ( ) ;
  AddToCyclicFlashTxt ( ( uint16_t ) OK ) ;
} /* AddToCyclicCommandReplyOK */


static void AddToCyclicCommandReplyFail ( void )
{
  /*----------------------------------------------------------------*/
  AddToCyclicReplyDevicesID ( ) ;
  AddToCyclicReplyForCommand ( ) ;
  AddToCyclicFlashTxt ( ( uint16_t ) Fail ) ;
} /* AddToCyclicCommandReplyFail */


static void ConnectCommandExecute ( void )
{
  uint8_t Ch ;
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  for ( Loop = 0 ; Loop <= PasswordSize ; Loop ++ )
  {
    Ch = EEBlock . Password [ Loop ] ;
    if ( SerialService . ParamText [ Loop ] != Ch )
      return ;
    if ( Ch == null )
      break ;
  } /* for */ ;
  LEDPort &= ~ ( 1 << ConnectLed ) ;
  ConnectContext = true ;
  AddToCyclicCommandReplyOK ( ) ;
} /* ConnectCommandExecute */


static void DisConnectCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  ConnectContext = false ;
  AddToCyclicCommandReplyOK ( ) ;
  LEDPort |= 1 << ConnectLed ;
} /* DisConnectCommandExecute */


static void SetDeviceNameCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  if ( ! SerialService . ParamText [ 0 ] )
  {
    AddToCyclicCommandReplyFail ( ) ;
    return ;
  } /* if */ ;
  Assign ( EEBlock . MyDeviceID ) ;
  AddToCyclicCommandReplyOK ( ) ;
} /* SetDeviceNameCommandExecute */


static void HelloCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  AddToCyclicReplyDevicesID ( ) ;
  AddToCyclicReplyForCommand ( ) ;
  AddToCyclicFlashTxt ( ( uint16_t ) DeviceName ) ;
} /* HelloCommandExecute */


static void GetVersionCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  AddToCyclicReplyDevicesID ( ) ;
  AddToCyclicReplyForCommand ( ) ;
  AddToCyclicFlashTxt ( ( uint16_t ) HardwareSoftwareVersion ) ;
} /* GetVersionCommandExecute */


static void GetIDCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  AddToCyclicReplyDevicesID ( ) ;
  AddToCyclicReplyForCommand ( ) ;
  AddToCyclicTxt ( EEBlock . MyDeviceID ) ;
  AddToCyclic ( cr ) ;
} /* GetIDCommandExecute */


static void ReinitCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  EEBlockOperation ( ) ;
  AddToCyclicCommandReplyOK ( ) ;
} /* ReinitCommandExecute */


static void ResetEECommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  SetDefaultEEBlock ( ) ;
  AddToCyclicCommandReplyOK ( ) ;
} /* ResetEECommandExecute */


static void SaveEECommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  WriteEEBlock ( ) ;
  AddToCyclicCommandReplyOK ( ) ;
} /* SaveEECommandExecute */


static void AddSuperiorChnCommandExecute ( void )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  if ( ! SerialService . ParamText [ 0 ] )
  {
    AddToCyclicCommandReplyFail ( ) ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < EventDestinDeviceArraySize ; Loop ++ )
  {
    if ( ! EEBlock . EventDestinDeviceID [ Loop ] [ 0 ] )
    {
      Assign ( EEBlock . EventDestinDeviceID [ Loop ] ) ;
      AddToCyclicCommandReplyOK ( ) ;
      return ;
    } /* if */ ;
  } /* for */ ;
  AddToCyclicCommandReplyFail ( ) ;
} /* AddSuperiorChnCommandExecute */


static void DelSuperiorChnCommandExecute ( void )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  if ( ! SerialService . ParamText [ 0 ] )
  {
    AddToCyclicCommandReplyFail ( ) ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < EventDestinDeviceArraySize ; Loop ++ )
  {
    if ( CompareDeviceID ( EEBlock . EventDestinDeviceID [ Loop ] , SerialService . ParamText ) )
    {
      ClearBlock ( EEBlock . EventDestinDeviceID [ Loop ] , sizeof ( DeviceIDType ) ) ;
      AddToCyclicCommandReplyOK ( ) ;
      return ;
    } /* if */ ;
  } /* for */ ;
  AddToCyclicCommandReplyFail ( ) ;
} /* DelSuperiorChnCommandExecute */


static void ListSuperiorChnCommandExecute ( void )
{
  uint8_t Loop ;
  uint8_t EmptyTable ;
  /*----------------------------------------------------------------*/
  EmptyTable = true ;
  for ( Loop = 0 ; Loop < EventDestinDeviceArraySize ; Loop ++ )
  {
    if ( EEBlock . EventDestinDeviceID [ Loop ] [ 0 ] )
    {
      EmptyTable = false ;
      break ;
    } /* if */ ;
  } /* for */ ;
  if ( EmptyTable )
  {
    AddToCyclicReplyDevicesID ( ) ;
    AddToCyclicReplyForCommand ( ) ;
    AddToCyclicFlashTxt ( ( uint16_t ) EmptyFlagTxt ) ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < EventDestinDeviceArraySize ; Loop ++ )
  {
    if ( EEBlock . EventDestinDeviceID [ Loop ] [ 0 ] )
    {
      AddToCyclicReplyDevicesID ( ) ;
      AddToCyclicReplyForCommand ( ) ;
      AddToCyclicTxt ( EEBlock . EventDestinDeviceID [ Loop ] ) ;
      AddToCyclic ( cr ) ;
    } /* if */ ;
  } /* for */ ;
} /* ListSuperiorChnCommandExecute */


static void MeasServiceOnCommandExecute ( void )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  if ( ! SerialService . ParamText [ 0 ] )
  {
    AddToCyclicCommandReplyFail ( ) ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < TempEchoDevicesArraySize ; Loop ++ )
  {
    if ( ! TempEchoDevices [ Loop ] [ 0 ] )
    {
      Assign ( TempEchoDevices [ Loop ] ) ;
      AddToCyclicCommandReplyOK ( ) ;
      return ;
    } /* if */ ;
  } /* for */ ;
  AddToCyclicCommandReplyFail ( ) ;
} /* MeasServiceOnCommandExecute */


static void MeasServiceOffCommandExecute ( void )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  if ( ! SerialService . ParamText [ 0 ] )
  {
    AddToCyclicCommandReplyFail ( ) ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < TempEchoDevicesArraySize ; Loop ++ )
  {
    if ( CompareDeviceID ( TempEchoDevices [ Loop ] , SerialService . ParamText ) )
    {
      ClearBlock ( TempEchoDevices [ Loop ] , sizeof ( DeviceIDType ) ) ;
      AddToCyclicCommandReplyOK ( ) ;
      return ;
    } /* if */ ;
  } /* for */ ;
  AddToCyclicCommandReplyFail ( ) ;
} /* MeasServiceOffCommandExecute */


static void MeasServiceListCommandExecute ( void )
{
  uint8_t Loop ;
  uint8_t EmptyTable ;
  /*----------------------------------------------------------------*/
  EmptyTable = true ;
  for ( Loop = 0 ; Loop < TempEchoDevicesArraySize ; Loop ++ )
  {
    if ( TempEchoDevices [ Loop ] [ 0 ] )
    {
      EmptyTable = false ;
      break ;
    } /* if */ ;
  } /* for */ ;
  if ( EmptyTable )
  {
    AddToCyclicReplyDevicesID ( ) ;
    AddToCyclicReplyForCommand ( ) ;
    AddToCyclicFlashTxt ( ( uint16_t ) EmptyFlagTxt ) ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < TempEchoDevicesArraySize ; Loop ++ )
  {
    if ( TempEchoDevices [ Loop ] [ 0 ] )
    {
      AddToCyclicReplyDevicesID ( ) ;
      AddToCyclicReplyForCommand ( ) ;
      AddToCyclicTxt ( TempEchoDevices [ Loop ] ) ;
      AddToCyclic ( cr ) ;
    } /* if */ ;
  } /* for */ ;
} /* MeasServiceListCommandExecute */


static void NewPasswordCommandExecute ( void )
{
  uint8_t Ch ;
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  if ( ! SerialService . ParamText [ 0 ] )
  {
    for ( Loop = 0 ; Loop <= PasswordSize ; Loop ++ )
    {
      Ch = SerialService . ParamText [ Loop ] ;
      EEBlock . Password [ Loop ] = Ch ;
      if ( Ch == null )
        break ;
    } /* for */ ;
    AddToCyclicCommandReplyOK ( ) ;
    return ;
  } /* if */ ;
  AddToCyclicCommandReplyFail ( ) ;
} /* NewPasswordCommandExecute */


static void AddParamCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  if ( ! SerialService . ParamText [ 0 ] )
  {
    AddToCyclicCommandReplyFail ( ) ;
    return ;
  } /* if */ ;
  SerialService . ParamText [ ParamNameSize ] = 0 ;
  ClearBlock ( EEBlock . ParamName , ParamNameSize + 1 ) ;
  StrCopy ( EEBlock . ParamName , SerialService . ParamText ) ;
  AddToCyclicCommandReplyOK ( ) ;
} /* AddParamCommandExecute */


static void DelParamCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  ClearBlock ( EEBlock . ParamName , ParamNameSize + 1 ) ;
  AddToCyclicCommandReplyOK ( ) ;
} /* DelParamCommandExecute */


static void ListParamCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  if ( ! ConnectContext )
    return ;
  AddToCyclicReplyDevicesID ( ) ;
  AddToCyclicReplyForCommand ( ) ;
  if ( EEBlock . ParamName [ 0 ] )
  {
    AddToCyclicTxt ( EEBlock . ParamName ) ;
    AddToCyclic ( cr ) ;
  } /* if ... */
  else
  {
    AddToCyclicFlashTxt ( ( uint16_t ) EmptyFlagTxt ) ;
  } /* if ... else */ ;
} /* ListParamCommandExecute */


static void SendTempReplyData ( uint8_t * DestDeviceName )
{
  /*----------------------------------------------------------------*/
  if ( * DestDeviceName )
  {
    AddToCyclicTxt ( DestDeviceName ) ;
    AddToCyclic ( PartSeparator ) ;
    AddToCyclicTxt ( EEBlock . MyDeviceID ) ;
    AddToCyclic ( PartSeparator ) ;
    AddToCyclicReplyForCommand ( ) ;
    if ( ( ! EEBlock . ParamName [ 0 ] ) || ( ! SensorPresent ) )
    {
      AddToCyclicFlashTxt ( ( uint16_t ) Fail ) ;
      return ;
    } /* if */ ;
    AddToCyclicTxt ( EEBlock . ParamName ) ;
    AddToCyclic ( ParamSeparator ) ;
    AddToCyclicTxt ( ParamAsciiValue ) ;
    AddToCyclic ( cr ) ;
  } /* if */ ;
} /* SendTempReplyData */


static void AutoSendTempData ( uint8_t * DestDeviceName )
{
  /*----------------------------------------------------------------*/
  if ( * DestDeviceName )
  {
    AddToCyclicTxt ( DestDeviceName ) ;
    AddToCyclic ( PartSeparator ) ;
    AddToCyclicTxt ( EEBlock . MyDeviceID ) ;
    AddToCyclic ( PartSeparator ) ;
    AddToCyclicFlashTxt ( ( uint16_t ) AutoSendCommand ) ;
    AddToCyclic ( ParamSeparator ) ;
    if ( ( ! EEBlock . ParamName [ 0 ] ) || ( ! SensorPresent ) )
    {
      AddToCyclicFlashTxt ( ( uint16_t ) Fail ) ;
      return ;
    } /* if */ ;
    AddToCyclicTxt ( EEBlock . ParamName ) ;
    AddToCyclic ( ParamSeparator ) ;
    AddToCyclicTxt ( ParamAsciiValue ) ;
    AddToCyclic ( cr ) ;
  } /* if */ ;
} /* AutoSendTempData */


static void GetParamValueCommandExecute ( void )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  SendTempReplyData ( SerialService . SourceDeviceName ) ;
  for ( Loop = 0 ; Loop < EventDestinDeviceArraySize ; Loop ++ )
  {
    SendTempReplyData ( EEBlock . EventDestinDeviceID [ Loop ] ) ;
  } /* for */ ;
  for ( Loop = 0 ; Loop < TempEchoDevicesArraySize ; Loop ++ )
  {
    SendTempReplyData ( TempEchoDevices [ Loop ] ) ;
  } /* for */ ;
} /* GetParamValueCommandExecute */


static void AutoOnCommandExecute ( void )
{
  /*-------------------------------------------------------------------*/
  CurrentMode = 1 ;
  EEBlock . AutoMode = CurrentMode ;
  AddToCyclicCommandReplyOK ( ) ;
} /* AutoOnCommandExecute */


static void AutoOffCommandExecute ( void )
{
  /*-------------------------------------------------------------------*/
  CurrentMode = 0 ;
  EEBlock . AutoMode = CurrentMode ;
  AddToCyclicCommandReplyOK ( ) ;
} /* AutoOffCommandExecute */


static void GetModeCommandExecute ( void )
{
  /*-------------------------------------------------------------------*/
  AddToCyclicReplyDevicesID ( ) ;
  AddToCyclicReplyForCommand ( ) ;
  if ( CurrentMode )
    AddToCyclicFlashTxt ( ( uint16_t ) ModeIsOnTxt ) ;
  else
    AddToCyclicFlashTxt ( ( uint16_t ) ModeIsOffTxt ) ;
  AddToCyclic ( cr ) ;
} /* GetModeCommandExecute */


void _1WireDelay ( uint16_t DelayTime )
{
  uint16_t StartTimer ;
  uint16_t CurrTimer ;
  /*-------------------------------------------------------------------*/
  StartTimer = TCNT1 ;
  for ( ; ; )
  {
    CurrTimer = TCNT1 ;
    if ( ( CurrTimer - StartTimer ) >= DelayTime )
      break ;
  } /* for */ ;
} /* _1WireDelay */


void _1WireDelay5us ( void )
{
  uint8_t Loop ;
  /*-------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < TimerDelay5us ; Loop ++ )
    nop ( ) ;
} /* _1WireDelay5us */


void _1WireDelay15us ( void )
{
  uint8_t Loop ;
  /*-------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < TimerDelay15us ; Loop ++ )
    nop ( ) ;
} /* _1WireDelay15us */


void _1WireDelay45us ( void )
{
  uint8_t Loop ;
  /*-------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < TimerDelay45us ; Loop ++ )
    nop ( ) ;
} /* _1WireDelay45us */


void _1WireDelay60us ( void )
{
  uint8_t Loop ;
  /*-------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < TimerDelay60us ; Loop ++ )
    nop ( ) ;
} /* _1WireDelay60us */


static void _1WireTempSensorBitWrite ( uint8_t BitData )
{
  /*-------------------------------------------------------------------*/
  cli ( ) ;
  Set_1WireAsOutput ;
  Set_1WireLow ;
  _1WireDelay15us ( ) ;
  if ( BitData )
  {
    Set_1WireAsInput ;
  } /* if */ ;
  _1WireDelay60us ( ) ;
  Set_1WireAsInput ;
  sei ( ) ;
  _1WireDelay1us ( ) ;
} /* _1WireTempSensorBitWrite */


static uint8_t _1WireTempSensorReset ( void )
{
  uint8_t PortData ;
  /*-------------------------------------------------------------------*/
  wdt_reset ( ) ;
  Set_1WireAsOutput ;
  Set_1WireLow ;
  _1WireDelay ( TimerDelay480us ) ;
  wdt_reset ( ) ;
  Set_1WireAsInput ;
  _1WireDelay60us ( ) ;
  PortData = Get_1WirePin ;
  _1WireDelay ( TimerDelay150us ) ;
  wdt_reset ( ) ;
  return ( PortData ) ;
} /* _1WireTempSensorReset */


static uint8_t _1WireTempSensorPresent ( void )
{
  /*-------------------------------------------------------------------*/
  if ( _1WireTempSensorReset ( ) )
    return ( false ) ;
  else
    return ( true ) ;
} /* _1WireTempSensorPresent */


static void _1WireTempSensorWrite ( uint8_t Data )
{
  uint8_t Loop ;
  /*-------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < 8 ; Loop ++ )
  {
    wdt_reset ( ) ;
    _1WireTempSensorBitWrite ( Data & 0x01 ) ;
    Data = Data >> 1 ;
  } /* for */ ;
  _1WireDelay5us ( ) ;
} /* _1WireTempSensorWrite */


static uint8_t _1WireTempSensorBitRead ( void )
{
  uint8_t PortData ;
  /*-------------------------------------------------------------------*/
  cli ( ) ;
  Set_1WireAsOutput ;
  Set_1WireLow ;
  _1WireDelay1us ( ) ;
  Set_1WireAsInput ;
  _1WireDelay15us ( ) ;
  PortData = Get_1WirePin ;
  sei ( ) ;
  _1WireDelay45us ( ) ;
  if ( PortData )
    return ( 1 ) ;
  else
    return ( 0 ) ;
} /* _1WireTempSensorBitRead */


static uint8_t _1WireTempSensorRead ( void )
{
  uint8_t Data ;
  uint8_t Loop ;
  /*-------------------------------------------------------------------*/
  Data = 0 ;
  for ( Loop = 0 ; Loop < 8 ; Loop ++ )
  {
    wdt_reset ( ) ;
    Data = Data >> 1 ;
    if ( _1WireTempSensorBitRead ( ) )
      Data |= 0x80 ;
  } /* if */ ;
  return ( Data ) ;
} /* _1WireTempSensorRead */


static int16_t Read_1WireTemp ( void )
{
  int16_t TempValue ;
  uint16_t WorkTemp ;
  uint8_t LSBData ;
  uint8_t MSBData ;
  /*-------------------------------------------------------------------*/
  _1WireTempSensorReset ( ) ;
  _1WireTempSensorWrite ( 0xCC ) ;
  _1WireTempSensorWrite ( 0xBE ) ;
  LSBData = _1WireTempSensorRead ( ) ;
  MSBData = _1WireTempSensorRead ( ) ;
  WorkTemp = ( MSBData << 8 ) | LSBData ;
  TempValue = ( int16_t ) WorkTemp ;
  return ( TempValue ) ;
} /* Read_1WireTemp */


static uint8_t Conv_1WireTemp ( void )
{
  /*-------------------------------------------------------------------*/
  if ( ! _1WireTempSensorPresent ( ) )
  {
    LEDPort &= ~ ( 1 << SensorFailLed ) ;
    return ( false ) ;
  } /* if */ ;
  LEDPort |= 1 << SensorFailLed ;
  _1WireTempSensorWrite ( 0xCC ) ;
  _1WireTempSensorWrite ( 0x44 ) ;
  return ( true ) ;
} /* Conv_1WireTemp */


static void AddToAsciiValue ( uint8_t Ch )
{
  /*-------------------------------------------------------------------*/
  if ( ParamAsciiValueIndex < ParamAsciiValueSize )
  {
    ParamAsciiValue [ ParamAsciiValueIndex ] = Ch ;
    ParamAsciiValueIndex ++ ;
    ParamAsciiValue [ ParamAsciiValueIndex ] = null ;
  } /* if */ ;
} /* AddToAsciiValue */


static void WordToStr ( uint8_t * Number ,
                        uint8_t   ConvWidth ,
                        uint8_t   EmptyCh ,
                        uint16_t  Value )
{
  uint8_t Loop ;
  uint8_t LocalBuffer [ 8 ] ;
  uint8_t LastDigit ;
  /*-------------------------------------------------------------------*/
  if ( ConvWidth > 7 )
  {
    * Number ++ = '!' ;
    * Number = 0 ;
    return ;
  } /* if */ ;
  for ( Loop = 0 ; Loop < ConvWidth ; Loop ++ )
    LocalBuffer [ Loop ] = EmptyCh ;
  LocalBuffer [ ConvWidth ] = 0 ;
  Loop = ConvWidth - 1 ;
  for ( ; ; )
  {
    LastDigit = ( uint8_t ) ( Value % 10 ) ;
    LocalBuffer [ Loop ] = LastDigit + 0x30 ;
    Value = Value / 10 ;
    if ( ( ! Loop ) || ( ! Value ) )
      break ;
    Loop -- ;
  } /* for */ ;
  for ( Loop = 0 ; Loop <= ConvWidth ; Loop ++ )
    * Number ++ = LocalBuffer [ Loop ] ;
} /* WordToStr */


static void ConvToAscii ( void )
{
  uint8_t Number [ 6 ] ;
  uint16_t Temp ;
  /*-------------------------------------------------------------------*/
  ParamAsciiValueIndex = 0 ;
  if ( SensorPresent )
  {
    if ( TempDataValid )
    {
      if ( CurrentTemp < 0 )
      {
        Temp = ( uint16_t ) ( - CurrentTemp ) ;
        AddToAsciiValue ( '-' ) ;
      } /* if ... */
      else
      {
        Temp = ( uint16_t ) CurrentTemp ;
      } /* if ... else */ ;
      Temp *= 10 ;
      Temp /= 16 ;
      WordToStr ( Number , 4 , '0' , Temp ) ;
      if ( Number [ 0 ] == '0' )
      {
        if ( Number [ 1 ] != '0' )
        {
          AddToAsciiValue ( Number [ 1 ] ) ;
        } /* if */ ;
      } /* if ... */
      else
      {
        AddToAsciiValue ( Number [ 0 ] ) ;
        AddToAsciiValue ( Number [ 1 ] ) ;
      } /* if ... else */ ;
      AddToAsciiValue ( Number [ 2 ] ) ;
      AddToAsciiValue ( ',' ) ;
      AddToAsciiValue ( Number [ 3 ] ) ;
      return ;
    } /* if */ ;
  } /* if */ ;
} /* ConvToAscii */


static void _1WirePool ( void )
{
  /*-------------------------------------------------------------------*/
  if ( _1WireEvent )
  {
    _1WireEvent = false ;
    switch ( _1WireAutomatState )
    {
      case 0 :
        SensorPresent = Conv_1WireTemp ( ) ;
        if ( ! SensorPresent )
          TempDataValid = false ;
        _1WireAutomatState = 1 ;
        break ;
      case 1 :
        if ( SensorPresent )
        {
          CurrentTemp = Read_1WireTemp ( ) ;
          ConvToAscii ( ) ;
          TempDataValid = true ;
        } /* if */ ;
        _1WireAutomatState = 0 ;
        break ;
    } /* switch */
  } /* if */ ;
} /* _1WirePool */


static void ReplyCommandExecute ( void )
{
  /*----------------------------------------------------------------*/
  if ( CyclicDataPresent ( ) )
  {
    FlushCyclic ( ) ;
  } /* if ... */
  else
  {
    SendSerialTxt ( SerialService . SourceDeviceName ) ;
    SendSerial ( PartSeparator ) ;
    SendSerialTxt ( SerialService . DestinationDeviceName ) ;
    SendSerial ( PartSeparator ) ;
    SendSerialFlashTxt ( ( uint16_t ) NoDataText ) ;
  } /* if ... else */ ;
} /* ReplyCommandExecute */


static void ExecuteAssociatonCommand ( uint8_t * String ,
                                       uint8_t   AssociationTableSize ,
                                       uint16_t  AssociationTable )
{
  uint8_t Loop ;
  CommandNameAssociationRecT AssociationElement ;
  uint16_t MemAddress ;
  /*----------------------------------------------------------------*/
  MemAddress = AssociationTable ;
  for ( Loop = 0 ; Loop < AssociationTableSize ; Loop ++ )
  {
    AssociationElement . CommandService = ( CommandExecuteServiceProcT ) pgm_read_word_near ( MemAddress ) ;
    MemAddress += 2 ;
    AssociationElement . CommandName = ( uint8_t * ) pgm_read_word_near ( MemAddress ) ;
    MemAddress += 2 ;
    if ( CompareFlash ( String , ( uint16_t ) AssociationElement . CommandName ) )
    {
      AssociationElement . CommandService ( ) ;
      return ;
    } /* if */ ;
  } /* for */ ;
} /* ExecuteAssociatonCommand */


static void ExecuteCommand ( void )
{
  uint8_t FrameToMe ;
  /*----------------------------------------------------------------*/
  FrameToMe = MeDestination ( ) ;
  if ( ! FrameToMe )
  {
    FrameToMe = GeneralDestination ( ) ;
    if ( FrameToMe )
      StrCopy ( SerialService . DestinationDeviceName ,
                EEBlock . MyDeviceID ) ;
  } /* if */ ;
  if ( FrameToMe )
  {
    ExecuteAssociatonCommand ( SerialService . CommandText ,
                               CommandNameAssociationTabeSize ,
                               ( uint16_t ) CommandNameAssociationTabe ) ;
  } /* if */ ;
} /* ExecuteCommand */


static void ResetOperation ( void ) ;

static void DestinationIDPutCh ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
  if ( SerialService . DestDeviceNameIndex < DeviceIDSize )
  {
    SerialService . DestinationDeviceName [ SerialService . DestDeviceNameIndex ++ ] = Ch ;
  } /* if */ ;
} /* DestinationIDPutCh */


static void SourceIDPutCh ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
  if ( SerialService . SrcDeviceNameIndex < DeviceIDSize )
  {
    SerialService . SourceDeviceName [ SerialService . SrcDeviceNameIndex ++ ] = Ch ;
  } /* if */ ;
} /* SourceIDPutCh */


static void CommandIDPutCh ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
  if ( SerialService . CommandTextIndex < CommandBufferSize )
  {
    SerialService . CommandText [ SerialService . CommandTextIndex ++ ] = Ch ;
  } /* if */ ;
} /* CommandIDPutCh */


static void ParamPutCh ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
  if ( SerialService . ParamTextIndex < ParamBufferSize )
  {
    SerialService . ParamText [ SerialService . ParamTextIndex ++ ] = Ch ;
  } /* if */ ;
} /* ParamPutCh */


static void SelectParam ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  if ( SerialDataPresent ( ) )
  {
    Ch = GetSerial ( ) ;
    if ( Ch == cr )
    {
      ParamPutCh ( null ) ;
      ExecuteCommand ( ) ;
      ResetOperation ( ) ;
      return ;
    } /* if */;
    if ( Ch < Space )
    {
      ResetOperation ( ) ;
      return ;
    } /* if */ ;
    ParamPutCh ( Ch ) ;
  } /* if */ ;
} /* SelectParam */


static void SelectCommand ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  if ( SerialDataPresent ( ) )
  {
    Ch = GetSerial ( ) ;
    if ( Ch == cr )
    {
      CommandIDPutCh ( null ) ;
      ExecuteCommand ( ) ;
      ResetOperation ( ) ;
      return ;
    } /* if */;
    if ( Ch < Space )
    {
      ResetOperation ( ) ;
      return ;
    } /* if */ ;
    if ( Ch == ParamSeparator )
    {
      CommandIDPutCh ( null ) ;
      SerialService . SerialService = SelectParam ;
      return ;
    } /* if */;
    CommandIDPutCh ( Ch ) ;
  } /* if */ ;
} /* SelectCommand */


static void SelectSourceService ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  if ( SerialDataPresent ( ) )
  {
    Ch = GetSerial ( ) ;
    if ( Ch < Space )
    {
      ResetOperation ( ) ;
      return ;
    } /* if */ ;
    if ( Ch == PartSeparator )
    {
      SourceIDPutCh ( null ) ;
      SerialService . SerialService = SelectCommand ;
      return ;
    } /* if */;
    SourceIDPutCh ( Ch ) ;
  } /* if */ ;
} /* SelectSourceService */


static void SelectDestOperation ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  if ( SerialDataPresent ( ) )
  {
    Ch = GetSerial ( ) ;
    if ( Ch < Space )
    {
      ResetOperation ( ) ;
      return ;
    } /* if */ ;
    if ( Ch == PartSeparator )
    {
      DestinationIDPutCh ( null ) ;
      SerialService . SerialService = SelectSourceService ;
      return ;
    } /* if */;
    DestinationIDPutCh ( Ch ) ;
  } /* if */ ;
} /* SelectDestOperation */


static void ResetOperation ( void )
{
  /*----------------------------------------------------------------*/
  ClearBlock ( SerialService . CommandText , sizeof ( CommandBufferType ) ) ;
  ClearBlock ( SerialService . ParamText , sizeof ( ParamBufferType ) ) ;
  ClearBlock ( SerialService . DestinationDeviceName , sizeof ( DeviceIDType ) ) ;
  ClearBlock ( SerialService . SourceDeviceName , sizeof ( DeviceIDType ) ) ;
  SerialService . CommandTextIndex = 0 ;
  SerialService . ParamTextIndex = 0 ;
  SerialService . DestDeviceNameIndex = 0 ;
  SerialService . SrcDeviceNameIndex = 0 ;
  SerialService . SerialService = SelectDestOperation ;
} /* ResetOperation */


static void SendYourselfData ( void )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  if ( ! CyclicDataPresent ( ) )
  {
    for ( Loop = 0 ; Loop < EventDestinDeviceArraySize ; Loop ++ )
    {
      AutoSendTempData ( EEBlock . EventDestinDeviceID [ Loop ] ) ;
    } /* for */ ;
    for ( Loop = 0 ; Loop < TempEchoDevicesArraySize ; Loop ++ )
    {
      AutoSendTempData ( TempEchoDevices [ Loop ] ) ;
    } /* for */ ;
    AutoSendService = ( CommandExecuteServiceProcT ) 0 ;
  } /* if ... else */ ;
} /* SendYourselfData */


static void TimeEventNotify ( void )
{
  /*----------------------------------------------------------------*/
  AutoDelayCounter -- ;
  if ( ! AutoDelayCounter )
  {
    AutoSendService = ( CommandExecuteServiceProcT ) 0 ;
    if ( CurrentMode )
    {
      AutoSendService = SendYourselfData ;
    } /* if */ ;
    AutoDelayCounter = EEBlock . AutoMeansTimeInterval ;
  } /* if */ ;
} /* TimeEventNotify */


static void SoftwareInit ( void )
{
  uint8_t Loop ;
  /*----------------------------------------------------------------*/
  AutoSendService = ( CommandExecuteServiceProcT ) 0 ;
  ConnectContext = false ;
  TimerCounter = 0 ;
  CyclicBuffer . CycFr = 0 ;
  CyclicBuffer . CycTo = 0 ;
  SerialService . RXDRec . CycFr = 0 ;
  SerialService . RXDRec . CycTo = 0 ;
  SerialService . TXDRec . CycFr = 0 ;
  SerialService . TXDRec . CycTo = 0 ;
  SerialService . TXDRec . State = IdleState ;
  SerialService . SerialLEDDelay = 0 ;
  for ( Loop = 0 ; Loop < TempEchoDevicesArraySize ; Loop ++ )
  {
    ClearBlock ( TempEchoDevices [ Loop ] , sizeof ( DeviceIDType ) ) ;
  } /* for */ ;
  ParamAsciiValue [ 0 ] = null ;
  TimeEvent = false ;
  _1WireTimeDelay = 0 ;
  _1WireEvent = false ;
  CurrentMode = 0 ;
} /* SoftwareInit */


int main ( void )
{
  HardwareInit ( Speed38400bps ) ;
  EnvirInit ( ) ;
  SoftwareInit ( ) ;
  USARTReceiveMode ( ) ;
  EEBlockOperation ( ) ;
  CurrentMode = EEBlock . AutoMode ;
  if ( EEBlock . AutoMeansTimeInterval < 10 )
    EEBlock . AutoMeansTimeInterval = 10 ;
  AutoDelayCounter = EEBlock . AutoMeansTimeInterval ;
  wdt_enable ( WDTO_1S ) ;
  sei ( ) ;
  ResetOperation ( ) ;
  for ( ; ; )
  {
    wdt_reset ( ) ;
    if ( TimeEvent )
    {
      TimeEvent = false ;
      LEDPort ^= 1 << StatusLed ;
      TimeEventNotify ( ) ;
    } /* if */ ;
    if ( SerialService . SerialService )
      SerialService . SerialService ( ) ;
    _1WirePool ( ) ;
    if ( AutoSendService )
      AutoSendService ( ) ;
  } /* for */ ;
  return ( 0 ) ;
} /* main */



